@@ -13,13 +13,13 @@ pub unsafe trait IntoPkgAdd: fmt::Debug {
 
 unsafe impl<'a> IntoPkgAdd for Package<'a> {
     unsafe fn as_alpm_pkg_t(&self) -> *mut alpm_pkg_t {
-        self.pkg.pkg
+        self.pkg.as_ptr()
     }
     unsafe fn added(self) {}
 }
 unsafe impl<'a> IntoPkgAdd for LoadedPackage<'a> {
     unsafe fn as_alpm_pkg_t(&self) -> *mut alpm_pkg_t {
-        self.pkg.pkg
+        self.pkg.as_ptr()
     }
     unsafe fn added(self) {
         std::mem::forget(self);
@@ -28,7 +28,7 @@ unsafe impl<'a> IntoPkgAdd for LoadedPackage<'a> {
 
 impl Alpm {
     pub fn trans_add_pkg<P: IntoPkgAdd>(&self, pkg: P) -> Result<(), AddError<P>> {
-        let ret = unsafe { alpm_add_pkg(self.handle, pkg.as_alpm_pkg_t()) };
+        let ret = unsafe { alpm_add_pkg(self.as_ptr(), pkg.as_alpm_pkg_t()) };
         let ok = self.check_ret(ret);
         match ok {
             Ok(_) => {
