@@ -6,54 +6,50 @@ use crate::{
 use std::ffi::CString;
 use std::fmt;
 use std::ops::Deref;
+use std::ptr::NonNull;
 
 use alpm_sys::*;
 
 #[derive(Copy, Clone)]
-pub struct Db<'a> {
-    pub(crate) db: *mut alpm_db_t,
-    pub(crate) handle: &'a Alpm,
+#[doc(alias("repo", "repository"))]
+pub struct Db<'h> {
+    db: NonNull<alpm_db_t>,
+    pub(crate) handle: &'h Alpm,
 }
 
-impl<'a> fmt::Debug for Db<'a> {
+impl<'h> fmt::Debug for Db<'h> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_struct("Db").field("name", &self.name()).finish()
     }
 }
 
-pub struct DbMut<'a> {
-    pub(crate) inner: Db<'a>,
+pub struct DbMut<'h> {
+    pub(crate) inner: Db<'h>,
 }
 
-impl<'a> fmt::Debug for DbMut<'a> {
+impl<'h> fmt::Debug for DbMut<'h> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         fmt::Debug::fmt(&self.inner, f)
     }
 }
 
-impl<'a> Deref for DbMut<'a> {
-    type Target = Db<'a>;
+impl<'h> Deref for DbMut<'h> {
+    type Target = Db<'h>;
 
-    fn deref(&self) -> &Db<'a> {
+    fn deref(&self) -> &Db<'h> {
         &self.inner
     }
 }
 
-impl<'a> From<DbMut<'a>> for Db<'a> {
-    fn from(db: DbMut<'a>) -> Db<'a> {
-        db.inner
-    }
-}
-
 impl Alpm {
     pub fn register_syncdb<S: Into<Vec<u8>>>(&self, name: S, sig_level: SigLevel) -> Result<Db> {
         let name = CString::new(name).unwrap();
 
         let db =
-            unsafe { alpm_register_syncdb(self.handle, name.as_ptr(), sig_level.bits() as i32) };
+            unsafe { alpm_register_syncdb(self.as_ptr(), name.as_ptr(), sig_level.bits() as i32) };
 
         self.check_null(db)?;
-        Ok(Db { db, handle: self })
+        unsafe { Ok(Db::new(self, db)) }
     }
 
     pub fn register_syncdb_mut<S: Into<Vec<u8>>>(
@@ -66,105 +62,113 @@ impl Alpm {
     }
 
     pub fn unregister_all_syncdbs(&mut self) -> Result<()> {
-        self.check_ret(unsafe { alpm_unregister_all_syncdbs(self.handle) })
+        self.check_ret(unsafe { alpm_unregister_all_syncdbs(self.as_ptr()) })
     }
 }
 
-impl<'a> DbMut<'a> {
+impl<'h> DbMut<'h> {
     pub fn unregister(self) {
-        unsafe { alpm_db_unregister(self.db) };
+        unsafe { alpm_db_unregister(self.as_ptr()) };
     }
 
     pub fn add_server<S: Into<Vec<u8>>>(&self, server: S) -> Result<()> {
         let server = CString::new(server).unwrap();
-        let ret = unsafe { alpm_db_add_server(self.db, server.as_ptr()) };
+        let ret = unsafe { alpm_db_add_server(self.as_ptr(), server.as_ptr()) };
         self.handle.check_ret(ret)
     }
 
-    pub fn set_servers<L: IntoRawAlpmList<'a, String>>(&self, list: L) -> Result<()> {
+    pub fn set_servers<'a, L: IntoRawAlpmList<'a, String>>(&self, list: L) -> Result<()> {
         let list = unsafe { list.into_raw_alpm_list() };
-        let ret = unsafe { alpm_db_set_servers(self.db, alpm_list_strdup(list.list())) };
+        let ret = unsafe { alpm_db_set_servers(self.as_ptr(), list.list()) };
         self.handle.check_ret(ret)
     }
 
     pub fn remove_server<S: Into<Vec<u8>>>(&self, server: S) -> Result<()> {
         let server = CString::new(server).unwrap();
-        let ret = unsafe { alpm_db_remove_server(self.db, server.as_ptr()) };
+        let ret = unsafe { alpm_db_remove_server(self.as_ptr(), server.as_ptr()) };
         self.handle.check_ret(ret)
     }
 }
 
-impl<'a> Db<'a> {
-    pub fn name(&self) -> &'a str {
-        let name = unsafe { alpm_db_get_name(self.db) };
+impl<'h> Db<'h> {
+    pub(crate) unsafe fn new(handle: &Alpm, db: *mut alpm_db_t) -> Db {
+        Db {
+            handle,
+            db: NonNull::new_unchecked(db),
+        }
+    }
+
+    pub fn as_ptr(self) -> *mut alpm_db_t {
+        self.db.as_ptr()
+    }
+
+    pub fn name(&self) -> &'h str {
+        let name = unsafe { alpm_db_get_name(self.as_ptr()) };
         unsafe { from_cstr(name) }
     }
 
-    pub fn servers(&self) -> AlpmList<'a, &'a str> {
-        let list = unsafe { alpm_db_get_servers(self.db) };
-        AlpmList::from_parts(self.handle, list)
+    pub fn servers(&self) -> AlpmList<'h, &'h str> {
+        let list = unsafe { alpm_db_get_servers(self.as_ptr()) };
+        unsafe { AlpmList::from_parts(self.handle, list) }
     }
 
-    pub fn pkg<S: Into<Vec<u8>>>(&self, name: S) -> Result<Package<'a>> {
+    pub fn pkg<S: Into<Vec<u8>>>(&self, name: S) -> Result<Package<'h>> {
         let name = CString::new(name).unwrap();
-        let pkg = unsafe { alpm_db_get_pkg(self.db, name.as_ptr()) };
+        let pkg = unsafe { alpm_db_get_pkg(self.as_ptr(), name.as_ptr()) };
         self.handle.check_null(pkg)?;
         unsafe { Ok(Package::new(self.handle, pkg)) }
     }
 
     #[doc(alias = "pkgcache")]
-    pub fn pkgs(&self) -> AlpmList<'a, Package<'a>> {
-        let pkgs = unsafe { alpm_db_get_pkgcache(self.db) };
-        AlpmList::from_parts(self.handle, pkgs)
+    pub fn pkgs(&self) -> AlpmList<'h, Package<'h>> {
+        let pkgs = unsafe { alpm_db_get_pkgcache(self.as_ptr()) };
+        unsafe { AlpmList::from_parts(self.handle, pkgs) }
     }
 
-    pub fn group<S: Into<Vec<u8>>>(&self, name: S) -> Result<Group<'a>> {
+    pub fn group<S: Into<Vec<u8>>>(&self, name: S) -> Result<Group<'h>> {
         let name = CString::new(name).unwrap();
-        let group = unsafe { alpm_db_get_group(self.db, name.as_ptr()) };
+        let group = unsafe { alpm_db_get_group(self.as_ptr(), name.as_ptr()) };
         self.handle.check_null(group)?;
-        Ok(Group {
-            handle: self.handle,
-            inner: group,
-        })
+        unsafe { Ok(Group::new(self.handle, group)) }
     }
 
     pub fn set_usage(&self, usage: Usage) -> Result<()> {
-        let ret = unsafe { alpm_db_set_usage(self.db, usage.bits() as i32) };
+        let ret = unsafe { alpm_db_set_usage(self.as_ptr(), usage.bits() as i32) };
         self.handle.check_ret(ret)
     }
 
-    pub fn search<L>(&self, list: L) -> Result<AlpmListMut<'a, Package<'a>>>
+    pub fn search<L>(&self, list: L) -> Result<AlpmListMut<'h, Package<'h>>>
     where
-        L: IntoRawAlpmList<'a, String>,
+        L: IntoRawAlpmList<'h, String>,
     {
         let mut ret = std::ptr::null_mut();
         let list = unsafe { list.into_raw_alpm_list() };
-        let ok = unsafe { alpm_db_search(self.db, list.list(), &mut ret) };
+        let ok = unsafe { alpm_db_search(self.as_ptr(), list.list(), &mut ret) };
         self.handle.check_ret(ok)?;
-        Ok(AlpmListMut::from_parts(self.handle, ret))
+        unsafe { Ok(AlpmListMut::from_parts(self.handle, ret)) }
     }
 
     #[doc(alias = "groupcache")]
-    pub fn groups(&self) -> Result<AlpmListMut<'a, Group<'a>>> {
-        let groups = unsafe { alpm_db_get_groupcache(self.db) };
+    pub fn groups(&self) -> Result<AlpmListMut<'h, Group<'h>>> {
+        let groups = unsafe { alpm_db_get_groupcache(self.as_ptr()) };
         self.handle.check_null(groups)?;
-        Ok(AlpmListMut::from_parts(self.handle, groups))
+        unsafe { Ok(AlpmListMut::from_parts(self.handle, groups)) }
     }
 
     pub fn siglevel(&self) -> SigLevel {
-        let siglevel = unsafe { alpm_db_get_siglevel(self.db) };
+        let siglevel = unsafe { alpm_db_get_siglevel(self.as_ptr()) };
         SigLevel::from_bits(siglevel as u32).unwrap()
     }
 
     pub fn is_valid(&self) -> Result<()> {
-        let ret = unsafe { alpm_db_get_valid(self.db) };
+        let ret = unsafe { alpm_db_get_valid(self.as_ptr()) };
         self.handle.check_ret(ret)
     }
 
     pub fn usage(&self) -> Result<Usage> {
         let mut usage = 0;
 
-        let ret = unsafe { alpm_db_get_usage(self.db, &mut usage) };
+        let ret = unsafe { alpm_db_get_usage(self.as_ptr(), &mut usage) };
         self.handle.check_ret(ret)?;
 
         let usage = Usage::from_bits(usage as u32).unwrap();
