@@ -1,5 +1,5 @@
 use crate::utils::*;
-use crate::{free, Alpm, AlpmList, AlpmListMut, Db, IntoRawAlpmList, Package, Ver};
+use crate::{free, Alpm, AlpmList, AlpmListMut, Db, IntoRawAlpmList, Package, Pkg, Ver};
 
 use alpm_sys::alpm_depmod_t::*;
 use alpm_sys::*;
@@ -8,10 +8,11 @@ use std::ffi::{c_void, CString};
 use std::fmt;
 use std::marker::PhantomData;
 use std::mem::transmute;
+use std::ptr::NonNull;
 
 pub struct Dep<'a> {
-    pub(crate) inner: *mut alpm_depend_t,
-    pub(crate) phantom: PhantomData<&'a ()>,
+    inner: NonNull<alpm_depend_t>,
+    _marker: PhantomData<&'a ()>,
 }
 
 impl<'a> fmt::Debug for Dep<'a> {
@@ -36,7 +37,7 @@ pub struct Depend {
 
 impl Clone for Depend {
     fn clone(&self) -> Self {
-        let ptr = unsafe { alpm_dep_compute_string(self.inner) };
+        let ptr = unsafe { alpm_dep_compute_string(self.inner.as_ptr()) };
         assert!(!ptr.is_null(), "failed to compute string for dep");
         let dep = unsafe { alpm_dep_from_string(ptr) };
         assert!(!dep.is_null(), "failed to create dep from string");
@@ -88,7 +89,7 @@ impl<'a> AsDep for &Dep<'a> {
 
 impl Drop for Depend {
     fn drop(&mut self) {
-        unsafe { alpm_dep_free(self.dep.inner) }
+        unsafe { alpm_dep_free(self.dep.as_ptr()) }
     }
 }
 
@@ -104,7 +105,7 @@ impl<'a> PartialEq for Dep<'a> {
 impl<'a> fmt::Display for Dep<'a> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         unsafe {
-            let cs = alpm_dep_compute_string(self.inner);
+            let cs = alpm_dep_compute_string(self.as_ptr());
             assert!(!cs.is_null(), "failed to compute string for dep");
             let s = from_cstr(cs);
             let err = f.write_str(s);
@@ -117,7 +118,7 @@ impl<'a> fmt::Display for Dep<'a> {
 impl<'a> From<Dep<'a>> for Vec<u8> {
     fn from(dep: Dep<'a>) -> Vec<u8> {
         unsafe {
-            let cs = alpm_dep_compute_string(dep.inner);
+            let cs = alpm_dep_compute_string(dep.as_ptr());
             assert!(!cs.is_null(), "failed to compute string for dep");
             let s = std::ffi::CStr::from_ptr(cs);
             let s = s.to_bytes().to_vec();
@@ -132,59 +133,62 @@ impl Depend {
         let s = CString::new(s).unwrap();
         let dep = unsafe { alpm_dep_from_string(s.as_ptr()) };
         assert!(!dep.is_null(), "failed to create dep from string");
-
-        Depend {
-            dep: Dep {
-                inner: dep,
-                phantom: PhantomData,
-            },
+        unsafe {
+            Depend {
+                dep: Dep::from_ptr(dep),
+            }
         }
     }
 
     pub(crate) unsafe fn from_ptr(ptr: *mut alpm_depend_t) -> Depend {
         Depend {
-            dep: Dep {
-                inner: ptr,
-                phantom: PhantomData,
-            },
+            dep: Dep::from_ptr(ptr),
         }
     }
 }
 
 impl<'a> Dep<'a> {
-    pub fn dep(&self) -> Dep {
+    pub(crate) unsafe fn from_ptr<'b>(ptr: *mut alpm_depend_t) -> Dep<'b> {
         Dep {
-            inner: self.inner,
-            phantom: PhantomData,
+            inner: NonNull::new_unchecked(ptr),
+            _marker: PhantomData,
         }
     }
 
+    pub(crate) fn as_ptr(&self) -> *mut alpm_depend_t {
+        self.inner.as_ptr()
+    }
+
+    pub fn dep(&self) -> Dep {
+        unsafe { Dep::from_ptr(self.as_ptr()) }
+    }
+
     pub fn to_depend(&self) -> Depend {
         Depend::new(self.to_string())
     }
 
     pub fn name(&self) -> &'a str {
-        unsafe { from_cstr((*self.inner).name) }
+        unsafe { from_cstr((*self.as_ptr()).name) }
     }
 
     pub fn version(&self) -> Option<&'a Ver> {
-        unsafe { (*self.inner).version.as_ref().map(|p| Ver::from_ptr(p)) }
+        unsafe { (*self.as_ptr()).version.as_ref().map(|p| Ver::from_ptr(p)) }
     }
 
     unsafe fn version_unchecked(&self) -> &'a Ver {
-        Ver::from_ptr((*self.inner).version)
+        Ver::from_ptr((*self.as_ptr()).version)
     }
 
     pub fn desc(&self) -> Option<&'a str> {
-        unsafe { from_cstr_optional((*self.inner).desc) }
+        unsafe { from_cstr_optional((*self.as_ptr()).desc) }
     }
 
     pub fn name_hash(&self) -> u64 {
-        unsafe { (*self.inner).name_hash as u64 }
+        unsafe { (*self.as_ptr()).name_hash as u64 }
     }
 
     pub fn depmod(&self) -> DepMod {
-        unsafe { transmute::<alpm_depmod_t, DepMod>((*self.inner).mod_) }
+        unsafe { transmute::<alpm_depmod_t, DepMod>((*self.as_ptr()).mod_) }
     }
 
     pub fn depmodver(&self) -> DepModVer {
@@ -199,12 +203,6 @@ impl<'a> Dep<'a> {
             }
         }
     }
-    pub(crate) unsafe fn from_ptr(ptr: *mut alpm_depend_t) -> Dep<'a> {
-        Dep {
-            inner: ptr,
-            phantom: PhantomData,
-        }
-    }
 }
 
 #[derive(Debug, Eq, PartialEq, Copy, Clone, PartialOrd)]
@@ -251,8 +249,8 @@ unsafe impl<'a> Send for DepMissing<'a> {}
 unsafe impl<'a> Sync for DepMissing<'a> {}
 
 pub struct DepMissing<'a> {
-    pub(crate) inner: *mut alpm_depmissing_t,
-    pub(crate) phantom: PhantomData<&'a ()>,
+    inner: NonNull<alpm_depmissing_t>,
+    _marker: PhantomData<&'a ()>,
 }
 
 impl<'a> fmt::Debug for DepMissing<'a> {
@@ -285,24 +283,35 @@ impl fmt::Debug for DependMissing {
 
 impl Drop for DependMissing {
     fn drop(&mut self) {
-        unsafe { alpm_depmissing_free(self.inner.inner) }
+        unsafe { alpm_depmissing_free(self.inner.as_ptr()) }
     }
 }
 
 impl<'a> DepMissing<'a> {
+    pub(crate) unsafe fn from_ptr<'b>(ptr: *mut alpm_depmissing_t) -> DepMissing<'b> {
+        DepMissing {
+            inner: NonNull::new_unchecked(ptr),
+            _marker: PhantomData,
+        }
+    }
+
+    pub(crate) fn as_ptr(&self) -> *mut alpm_depmissing_t {
+        self.inner.as_ptr()
+    }
+
     pub fn target(&self) -> &str {
-        let target = unsafe { (*self.inner).target };
+        let target = unsafe { (*self.as_ptr()).target };
         unsafe { from_cstr(target) }
     }
 
     pub fn depend(&self) -> Dep {
-        let depend = unsafe { (*self.inner).depend };
+        let depend = unsafe { (*self.as_ptr()).depend };
 
         unsafe { Dep::from_ptr(depend) }
     }
 
     pub fn causing_pkg(&self) -> Option<&str> {
-        let causing_pkg = unsafe { (*self.inner).causingpkg };
+        let causing_pkg = unsafe { (*self.as_ptr()).causingpkg };
         if causing_pkg.is_null() {
             None
         } else {
@@ -315,7 +324,7 @@ impl<'a> AlpmList<'a, Db<'a>> {
     pub fn find_satisfier<S: Into<Vec<u8>>>(&self, dep: S) -> Option<Package<'a>> {
         let dep = CString::new(dep).unwrap();
 
-        let pkg = unsafe { alpm_find_dbs_satisfier(self.handle.handle, self.list, dep.as_ptr()) };
+        let pkg = unsafe { alpm_find_dbs_satisfier(self.handle.as_ptr(), self.list, dep.as_ptr()) };
         self.handle.check_null(pkg).ok()?;
         unsafe { Some(Package::new(self.handle, pkg)) }
     }
@@ -334,9 +343,9 @@ impl<'a> AlpmList<'a, Package<'a>> {
 impl Alpm {
     pub fn check_deps<'a>(
         &self,
-        pkgs: impl IntoRawAlpmList<'a, Package<'a>>,
-        rem: impl IntoRawAlpmList<'a, Package<'a>>,
-        upgrade: impl IntoRawAlpmList<'a, Package<'a>>,
+        pkgs: impl IntoRawAlpmList<'a, Pkg<'a>>,
+        rem: impl IntoRawAlpmList<'a, Pkg<'a>>,
+        upgrade: impl IntoRawAlpmList<'a, Pkg<'a>>,
         reverse_deps: bool,
     ) -> AlpmListMut<DependMissing> {
         let reverse_deps = if reverse_deps { 1 } else { 0 };
@@ -347,14 +356,14 @@ impl Alpm {
 
         let ret = unsafe {
             alpm_checkdeps(
-                self.handle,
+                self.as_ptr(),
                 pkgs.list(),
                 rem.list(),
                 upgrade.list(),
                 reverse_deps,
             )
         };
-        AlpmListMut::from_parts(self, ret)
+        unsafe { AlpmListMut::from_parts(self, ret) }
     }
 }
 
