@@ -93,28 +93,6 @@ where
     unsafe fn into_raw_alpm_list(self) -> RawAlpmList<'a, T, Self::Drop>;
 }
 
-impl<'a> IntoRawAlpmList<'a, Pkg<'a>> for AlpmList<'a, LoadedPackage<'a>> {
-    type Drop = False;
-    unsafe fn into_raw_alpm_list(self) -> RawAlpmList<'a, Pkg<'a>, Self::Drop> {
-        RawAlpmList {
-            list: self.list,
-            _marker1: PhantomData,
-            _marker2: PhantomData,
-        }
-    }
-}
-
-impl<'a> IntoRawAlpmList<'a, Pkg<'a>> for AlpmList<'a, Package<'a>> {
-    type Drop = False;
-    unsafe fn into_raw_alpm_list(self) -> RawAlpmList<'a, Pkg<'a>, Self::Drop> {
-        RawAlpmList {
-            list: self.list,
-            _marker1: PhantomData,
-            _marker2: PhantomData,
-        }
-    }
-}
-
 impl<'a, T> IntoRawAlpmList<'a, T> for AlpmList<'a, T>
 where
     T: AsAlpmListItemPtr<'a>,
@@ -287,7 +265,7 @@ where
 
 impl<'a> AlpmList<'a, String> {
     pub fn as_str<'b>(&'b self) -> AlpmList<'a, &'b str> {
-        AlpmList::from_parts(self.handle, self.list)
+        unsafe { AlpmList::from_parts(self.handle, self.list) }
     }
 }
 
@@ -299,7 +277,7 @@ where
     pub fn to_list_mut(&self) -> AlpmListMut<'a, T> {
         let list = unsafe { alpm_list_copy(self.list) };
         AlpmListMut {
-            list: AlpmList::from_parts(self.handle, list),
+            list: unsafe { AlpmList::from_parts(self.handle, list) },
         }
     }
 }
@@ -387,7 +365,7 @@ where
         self.list.list = unsafe { alpm_list_remove_item(self.list.list, item) };
         unsafe { (*item).next = ptr::null_mut() };
         unsafe { (*item).prev = ptr::null_mut() };
-        AlpmListMut::from_parts(self.handle, item)
+        unsafe { AlpmListMut::from_parts(self.handle, item) }
     }
 
     pub fn as_list(&self) -> AlpmList<'a, T> {
@@ -526,7 +504,7 @@ where
     for<'b> T: IntoAlpmListItem<'a, 'b>,
 {
     fn drop(&mut self) {
-        AlpmListMut::<T>::from_parts(self.list.handle, self.current);
+        unsafe { AlpmListMut::<T>::from_parts(self.list.handle, self.current) };
     }
 }
 
@@ -627,7 +605,7 @@ where
 }
 
 impl<'a, T> AlpmList<'a, T> {
-    pub(crate) fn from_parts(handle: &'a Alpm, list: *mut alpm_list_t) -> AlpmList<'a, T> {
+    pub(crate) unsafe fn from_parts(handle: &'a Alpm, list: *mut alpm_list_t) -> AlpmList<'a, T> {
         AlpmList {
             handle,
             list,
@@ -642,11 +620,14 @@ where
 {
     pub fn new(handle: &'a Alpm) -> AlpmListMut<'a, T> {
         AlpmListMut {
-            list: AlpmList::from_parts(handle, ptr::null_mut()),
+            list: unsafe { AlpmList::from_parts(handle, ptr::null_mut()) },
         }
     }
 
-    pub(crate) fn from_parts(handle: &'a Alpm, list: *mut alpm_list_t) -> AlpmListMut<'a, T> {
+    pub(crate) unsafe fn from_parts(
+        handle: &'a Alpm,
+        list: *mut alpm_list_t,
+    ) -> AlpmListMut<'a, T> {
         AlpmListMut {
             list: AlpmList::from_parts(handle, list),
         }
@@ -657,15 +638,23 @@ unsafe impl<'a> AsAlpmListItemPtr<'a> for Pkg<'a> {
     type Output = Pkg<'a>;
 
     fn as_ptr(&self) -> *mut c_void {
-        self.pkg as *mut c_void
+        (*self).as_ptr() as *mut c_void
     }
 }
 
 unsafe impl<'a> AsAlpmListItemPtr<'a> for Package<'a> {
-    type Output = Package<'a>;
+    type Output = Pkg<'a>;
 
     fn as_ptr(&self) -> *mut c_void {
-        self.pkg.pkg as *mut c_void
+        self.pkg.as_ptr() as *mut c_void
+    }
+}
+
+unsafe impl<'a> AsAlpmListItemPtr<'a> for LoadedPackage<'a> {
+    type Output = Pkg<'a>;
+
+    fn as_ptr(&self) -> *mut c_void {
+        self.pkg.as_ptr() as *mut c_void
     }
 }
 
@@ -673,7 +662,7 @@ unsafe impl<'a> AsAlpmListItemPtr<'a> for Db<'a> {
     type Output = Db<'a>;
 
     fn as_ptr(&self) -> *mut c_void {
-        self.db as *mut c_void
+        (*self).as_ptr() as *mut c_void
     }
 }
 
@@ -681,7 +670,7 @@ unsafe impl<'a> AsAlpmListItemPtr<'a> for Depend {
     type Output = Dep<'a>;
 
     fn as_ptr(&self) -> *mut c_void {
-        self.inner as *mut c_void
+        Dep::as_ptr(self) as *mut c_void
     }
 }
 
@@ -697,7 +686,7 @@ unsafe impl<'a> AsAlpmListItemPtr<'a> for Dep<'a> {
     type Output = Dep<'a>;
 
     fn as_ptr(&self) -> *mut c_void {
-        self.inner as *mut c_void
+        Dep::as_ptr(self) as *mut c_void
     }
 }
 
@@ -726,28 +715,44 @@ unsafe impl<'a> Push<'a> for Depend {}
 unsafe impl<'a> Push<'a> for Dep<'a> {}
 
 unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for Package<'a> {
-    type Borrow = Self;
+    type Borrow = Pkg<'a>;
     unsafe fn ptr_into_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self {
         Package::new(handle, ptr as *mut alpm_pkg_t)
     }
     unsafe fn ptr_as_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
-        Package::new(handle, ptr as *mut alpm_pkg_t)
+        Package::new(handle, ptr as *mut alpm_pkg_t).pkg
+    }
+}
+
+unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for Pkg<'a> {
+    type Borrow = Self;
+    unsafe fn ptr_into_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self {
+        Package::new(handle, ptr as *mut alpm_pkg_t).pkg
+    }
+    unsafe fn ptr_as_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
+        Package::new(handle, ptr as *mut alpm_pkg_t).pkg
+    }
+}
+
+unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for LoadedPackage<'a> {
+    type Borrow = Pkg<'a>;
+    unsafe fn ptr_into_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self {
+        LoadedPackage {
+            pkg: Package::new(handle, ptr as *mut alpm_pkg_t).pkg,
+        }
+    }
+    unsafe fn ptr_as_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
+        Package::new(handle, ptr as *mut alpm_pkg_t).pkg
     }
 }
 
 unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for Group<'a> {
     type Borrow = Self;
     unsafe fn ptr_into_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self {
-        Group {
-            inner: ptr as *mut alpm_group_t,
-            handle,
-        }
+        Group::new(handle, ptr as *mut alpm_group_t)
     }
     unsafe fn ptr_as_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
-        Group {
-            inner: ptr as *mut alpm_group_t,
-            handle,
-        }
+        Group::new(handle, ptr as *mut alpm_group_t)
     }
 }
 
@@ -775,14 +780,10 @@ unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for Dep<'a> {
 unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for Backup {
     type Borrow = Self;
     unsafe fn ptr_into_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self {
-        Backup {
-            inner: ptr as *mut alpm_backup_t,
-        }
+        Backup::from_ptr(ptr as *mut alpm_backup_t)
     }
     unsafe fn ptr_as_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
-        Backup {
-            inner: ptr as *mut alpm_backup_t,
-        }
+        Backup::from_ptr(ptr as *mut alpm_backup_t)
     }
 }
 
@@ -790,18 +791,12 @@ unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for OwnedFileConflict {
     type Borrow = FileConflict<'b>;
     unsafe fn ptr_into_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self {
         OwnedFileConflict {
-            inner: FileConflict {
-                inner: ptr as *mut alpm_fileconflict_t,
-                phantom: PhantomData,
-            },
+            inner: FileConflict::from_ptr(ptr as *mut alpm_fileconflict_t),
         }
     }
 
     unsafe fn ptr_as_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
-        FileConflict {
-            inner: ptr as *mut alpm_fileconflict_t,
-            phantom: PhantomData,
-        }
+        FileConflict::from_ptr(ptr as *mut alpm_fileconflict_t)
     }
 }
 
@@ -809,18 +804,12 @@ unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for DependMissing {
     type Borrow = DepMissing<'b>;
     unsafe fn ptr_into_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self {
         DependMissing {
-            inner: DepMissing {
-                inner: ptr as *mut alpm_depmissing_t,
-                phantom: PhantomData,
-            },
+            inner: DepMissing::from_ptr(ptr as *mut alpm_depmissing_t),
         }
     }
 
     unsafe fn ptr_as_alpm_list_item(_handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
-        DepMissing {
-            inner: ptr as *mut alpm_depmissing_t,
-            phantom: PhantomData,
-        }
+        DepMissing::from_ptr(ptr as *mut alpm_depmissing_t)
     }
 }
 
@@ -847,16 +836,10 @@ unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for Conflict<'a> {
 unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for Db<'a> {
     type Borrow = Self;
     unsafe fn ptr_into_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self {
-        Db {
-            db: ptr as *mut alpm_db_t,
-            handle,
-        }
+        Db::new(handle, ptr as *mut alpm_db_t)
     }
     unsafe fn ptr_as_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
-        Db {
-            db: ptr as *mut alpm_db_t,
-            handle,
-        }
+        Db::new(handle, ptr as *mut alpm_db_t)
     }
 }
 
@@ -864,18 +847,12 @@ unsafe impl<'a, 'b> IntoAlpmListItem<'a, 'b> for DbMut<'a> {
     type Borrow = Self;
     unsafe fn ptr_into_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self {
         DbMut {
-            inner: Db {
-                db: ptr as *mut alpm_db_t,
-                handle,
-            },
+            inner: Db::new(handle, ptr as *mut alpm_db_t),
         }
     }
     unsafe fn ptr_as_alpm_list_item(handle: &'a Alpm, ptr: *mut c_void) -> Self::Borrow {
         DbMut {
-            inner: Db {
-                db: ptr as *mut alpm_db_t,
-                handle,
-            },
+            inner: Db::new(handle, ptr as *mut alpm_db_t),
         }
     }
 }
