@@ -13,6 +13,7 @@ use std::io::{self, Read};
 use std::marker::PhantomData;
 use std::mem::{transmute, ManuallyDrop};
 use std::os::raw::c_uchar;
+use std::ptr::NonNull;
 use std::slice;
 use std::{cmp::Ordering, ops::Deref};
 
@@ -851,12 +852,7 @@ impl<'a> ReplaceQuestion<'a> {
     }
 
     pub fn newdb(&self) -> Db {
-        unsafe {
-            Db {
-                db: (*self.inner).newdb,
-                handle: &self.handle,
-            }
-        }
+        unsafe { Db::new(&self.handle, (*self.inner).newdb) }
     }
 }
 
@@ -921,7 +917,7 @@ impl<'a> RemovePkgsQuestion<'a> {
 
     pub fn packages(&'a self) -> AlpmList<'a, Package> {
         let list = unsafe { (*self.inner).packages };
-        AlpmList::from_parts(&self.handle, list)
+        unsafe { AlpmList::from_parts(&self.handle, list) }
     }
 }
 
@@ -938,7 +934,7 @@ impl<'a> SelectProviderQuestion<'a> {
 
     pub fn providers(&self) -> AlpmList<Package> {
         let list = unsafe { (*self.inner).providers };
-        AlpmList::from_parts(&self.handle, list)
+        unsafe { AlpmList::from_parts(&self.handle, list) }
     }
 
     pub fn depend(&self) -> Dep {
@@ -980,7 +976,7 @@ impl<'a> ImportKeyQuestion<'a> {
 
 pub struct Group<'a> {
     pub(crate) handle: &'a Alpm,
-    pub(crate) inner: *mut alpm_group_t,
+    inner: NonNull<alpm_group_t>,
 }
 
 impl<'a> fmt::Debug for Group<'a> {
@@ -993,19 +989,30 @@ impl<'a> fmt::Debug for Group<'a> {
 }
 
 impl<'a> Group<'a> {
+    pub(crate) unsafe fn new<'b>(handle: &'b Alpm, ptr: *mut alpm_group_t) -> Group<'b> {
+        Group {
+            handle,
+            inner: NonNull::new_unchecked(ptr),
+        }
+    }
+
+    pub(crate) fn as_ptr(&self) -> *mut alpm_group_t {
+        self.inner.as_ptr()
+    }
+
     pub fn name(&self) -> &'a str {
-        unsafe { from_cstr((*self.inner).name) }
+        unsafe { from_cstr((*self.as_ptr()).name) }
     }
 
     pub fn packages(&self) -> AlpmList<'a, Package<'a>> {
-        let pkgs = unsafe { (*self.inner).packages };
-        AlpmList::from_parts(self.handle, pkgs)
+        let pkgs = unsafe { (*self.as_ptr()).packages };
+        unsafe { AlpmList::from_parts(self.handle, pkgs) }
     }
 }
 
 pub struct ChangeLog<'a> {
-    pub(crate) pkg: &'a Pkg<'a>,
-    pub(crate) stream: *mut c_void,
+    pub(crate) pkg: Pkg<'a>,
+    stream: NonNull<c_void>,
 }
 
 impl<'a> fmt::Debug for ChangeLog<'a> {
@@ -1016,7 +1023,7 @@ impl<'a> fmt::Debug for ChangeLog<'a> {
 
 impl<'a> Drop for ChangeLog<'a> {
     fn drop(&mut self) {
-        unsafe { alpm_pkg_changelog_close(self.pkg.pkg, self.stream) };
+        unsafe { alpm_pkg_changelog_close(self.pkg.as_ptr(), self.as_ptr()) };
     }
 }
 
@@ -1026,14 +1033,27 @@ impl<'a> Read for ChangeLog<'a> {
             alpm_pkg_changelog_read(
                 buf.as_mut_ptr() as *mut c_void,
                 buf.len(),
-                self.pkg.pkg,
-                self.stream,
+                self.pkg.as_ptr(),
+                self.as_ptr(),
             )
         };
         Ok(ret)
     }
 }
 
+impl<'a> ChangeLog<'a> {
+    pub(crate) unsafe fn new<'b>(pkg: Pkg<'b>, ptr: *mut c_void) -> ChangeLog<'b> {
+        ChangeLog {
+            pkg,
+            stream: NonNull::new_unchecked(ptr),
+        }
+    }
+
+    pub(crate) fn as_ptr(&self) -> *mut c_void {
+        self.stream.as_ptr()
+    }
+}
+
 #[derive(Debug, Eq, PartialEq, Copy, Clone, Ord, PartialOrd, Hash)]
 pub enum Match {
     No,
@@ -1058,8 +1078,10 @@ pub enum CommitResult<'a> {
     Ok,
 }
 
+// TODO: unsound: need lifetime on handle
+#[derive(Copy, Clone)]
 pub struct Backup {
-    pub(crate) inner: *mut alpm_backup_t,
+    inner: NonNull<alpm_backup_t>,
 }
 
 impl fmt::Debug for Backup {
@@ -1072,12 +1094,22 @@ impl fmt::Debug for Backup {
 }
 
 impl Backup {
+    pub(crate) unsafe fn from_ptr(ptr: *mut alpm_backup_t) -> Backup {
+        Backup {
+            inner: NonNull::new_unchecked(ptr),
+        }
+    }
+
+    pub(crate) fn as_ptr(&self) -> *mut alpm_backup_t {
+        self.inner.as_ptr()
+    }
+
     pub fn hash(&self) -> &str {
-        unsafe { from_cstr((*self.inner).hash) }
+        unsafe { from_cstr((*self.as_ptr()).hash) }
     }
 
     pub fn name(&self) -> &str {
-        unsafe { from_cstr((*self.inner).name) }
+        unsafe { from_cstr((*self.as_ptr()).name) }
     }
 }
 
@@ -1198,13 +1230,24 @@ pub enum DownloadResult {
 }
 
 pub struct Signature {
-    pub(crate) sig: *mut c_uchar,
-    pub(crate) len: usize,
+    sig: NonNull<c_uchar>,
+    len: usize,
 }
 
 impl Signature {
+    pub(crate) unsafe fn new(sig: *mut c_uchar, len: usize) -> Signature {
+        Signature {
+            sig: NonNull::new_unchecked(sig),
+            len,
+        }
+    }
+
+    pub(crate) fn as_ptr(&self) -> *mut c_uchar {
+        self.sig.as_ptr()
+    }
+
     pub fn sig(&self) -> &[u8] {
-        unsafe { slice::from_raw_parts(self.sig, self.len) }
+        unsafe { slice::from_raw_parts(self.as_ptr(), self.len) }
     }
 }
 
@@ -1224,6 +1267,6 @@ impl Deref for Signature {
 
 impl Drop for Signature {
     fn drop(&mut self) {
-        unsafe { crate::free(self.sig as _) }
+        unsafe { crate::free(self.as_ptr() as _) }
     }
 }
